CON
        _clkmode        = xtal1 + pll16x
        _xinfreq        = 5_000_000

        SDPinBase       = 0

        IR_CMD_READ      = 1
        IR_CMD_SEND      = 2
        IR_CMD_TRANSLATE = 3
        IR_CMD_SETTIM    = 4
        IR_CMD_GETTIM    = 5
        IR_CMD_SETPAR    = 6
        IR_CMD_SETFLAGS  = 7
        IR_CMD_CLRFLAGS  = 8

        IR_PAR_GAPTIM    = 0
        
        IR_FLAG_ONCE     = 1<<8
        IR_FLAG_CLRTT    = 2<<8
        IR_FLAG_WRTT     = 4<<8
                
        IR_FLG_ONCE     = 1
        IR_FLG_CLRTT    = 2
        IR_FLG_WRTT     = 4
                
        IR_InPin        = 13
        IR_OutClockPin  = 14
        IR_OutPin       = 15
        IR_InMask       = |<IR_InPin
        IR_OutClockMask = |<IR_OutClockPin
        IR_OutMask      = |<IR_OutPin

        BUF_SIZE        = 80

obj
   io: "CogOS_IO(Term)"
   ' sd: "fsrw"

var
  long stack[500]
  byte outStr[ 256 ]
  
pub main | timing, i, j, foundi, timmax, fst

  ' start the IO subsystem and show the address of
  ' the buffers
  io.start( 1, 1 )
  io.str( string("IR RC driver test",13) )
  printAdr

  par_buf := @tim_tab + @smpl_tab<<16
  par_buf[1]:=@trans_buf 
  printAdr
  
  ' wait for a keypress
  io.rx

  ' start the COG that's doing the IR work
  cognew( @ir_rc, @par_buf )

  repeat while par_buf <> 0
  printAdr

  ' show the empty buffers
  printSamples
  printTiming
  
  ' wait for a keypress
  io.rx

  fst:=0  
  ' irCmd( constant( IR_CMD_SETPAR | IR_PAR_GAPTIM | $300<<16 ))
  irCmd( constant( IR_CMD_SETFLAGS | IR_FLAG_ONCE | IR_FLAG_WRTT | IR_FLAG_CLRTT ))
  repeat
    io.str( string("read",13) )
    irCmd( IR_CMD_READ )
    irWaitDone
    if fst==0
      fst:=1
      printSamples
      
    'io.str( string("read done!",13) )

    'printSamples
    printTiming
    
    'io.str( string("translate",13) )
    irCmd( IR_CMD_TRANSLATE )
    irWaitDone
    'io.str( string("translate done!",13) )

    ' search the max
    timmax:=0
    repeat i from 0 to 19
      if timmax < tim_tab[ i ]
        timmax := tim_tab[ i ]
        foundi := i

    'io.str( string("Max. Symbol: ") )
    'io.dec( foundi )
    foundi := "a"+foundi
    if foundi>"j"
      foundi:=foundi+("A"-"a"-10)
    'io.out( foundi )
    'io.out( 13 )
      
    'io.out( 13 )
    j:=0
    repeat i from 0 to BUF_SIZE-1
      'io.out( trans_buf[ i ] )
      outStr[j]:=trans_buf[ i ]
      j++
      if trans_buf[i]==foundi
        outStr[j]:=0
        io.str( @outStr )
        io.out(" ")
        io.dec( j )

        sumUp
        io.out(" ")
        
        'io.out( 13 )
        j:=0
      ' if trans_buf[i]=="E" or trans_buf[i]=="F"
      
    waitcnt( clkfreq+cnt )
    
    'io.str( string(13,"done!",13) )
    'io.rx

  repeat

pri sumUp | x, c, sum
  x:=0
  sum:=0
  repeat while c:=outStr[x++]
    if c=>"a" and c=<"j"
      sum:=sum+tim_tab[ (c-"a") ]
    else
      sum:=sum+tim_tab[ (c-"A"+10) ]
  io.out(" ")
  io.dec( sum )
  io.out(" ")
  io.dec( (sum*50)/1000 )
  io.str( string("ms") )
    
pri printAdr
  io.hex( @par_buf, 8 )
  io.out( "[" )
  io.hex( par_buf, 8 )
  io.out( "," )
  io.hex( par_buf[1], 8 )
  io.out( "]" )
  io.out( " " )
  io.hex( @tim_tab, 4 )
  io.out( " " )
  io.hex( @smpl_tab, 4 )
  io.out( " " )
  io.hex( @trans_buf, 4 )
  io.out( 13 )

pri printSamples | i
  io.str( string("Sample Table:",13) )
  repeat i from 0 to 255
    io.hex( smpl_tab[ i ], 4 )
    io.out( " " )
    if (i//16)==15
      io.out( $0d )
  io.out( 13 )

pri printTiming | i
  io.str( string("Timing:",13) )
  repeat i from 0 to 19
    io.hex( tim_tab[i] , 4 )
    io.out( " " )
    if (i//16)==9
      io.out( $0d )
  io.out( 13 )

pri irCmd( cmd )
  repeat while par_buf
  par_buf := cmd

pri irWaitDone
  repeat while par_buf
    
dat
  org 16
' for start this parameter buffer points to the different tables
par_buf       long      
par_buf_wh    word      @tim_tab '+16
par_buf_wl    word      @smpl_tab '+16
' when starting the COG this is part of the par_buf pointing to translation buffer
' after startup it will be used as timing-table
tim_tab       word      @trans_buf, 0[19]
' sample buffer
smpl_tab      word      0[BUF_SIZE] '[256]
' translation of the recorded signal
trans_buf     byte      0[BUF_SIZE] '[256]

' used to translate the timing to a sequence of letters
lowercase     byte      "abcdefghij"
uppercase     byte      "ABCDEFGHIJ"
    
' This one generates the ~33kHz carrier signal.
dat
  org 0

tim
ir_out        or        dira, ir_clk_mask
              mov       tim, cnt
              add       tim, ir_wait

da_lp         waitcnt   tim, ir_wait
              xor       outa, ir_clk_mask
              jmp       #da_lp

ir_clk_mask   long      IR_OutClockMask
ir_wait       long      80_000_000 / 33_000 / 2


dat
' this is the driver for IR.
        org 0
              ' the initialization is done at the end
ir_command
ir_rc         jmp       #ir_init

' command loop - waits for commands in the parameter buffer
ir_loop       rdlong    ir_command, par WZ
        if_z  jmp       #ir_loop

              ' extract the first byte as command, so the
              ' rest might be used as other parameter
              mov       ir_temp, ir_command
              and       ir_temp, #$ff
              
              ' as long as there are only 3 commands, compare is fine
              cmp       ir_temp, #IR_CMD_READ WZ
        if_z  jmp       #ir_read
              cmp       ir_temp, #IR_CMD_SEND WZ
        if_z  jmp       #ir_send
              cmp       ir_temp, #IR_CMD_TRANSLATE WZ
        if_z  jmp       #ir_translate
              cmp       ir_temp, #IR_CMD_SETPAR WZ
        if_z  jmp       #ir_setpar
              cmp       ir_temp, #IR_CMD_SETFLAGS WZ
        if_z  jmp       #ir_setflag
              cmp       ir_temp, #IR_CMD_CLRFLAGS WZ
        if_z  jmp       #ir_clrflag
              cmp       ir_temp, #IR_CMD_SETTIM WZ
        if_z  jmp       #ir_rd_tim

              ' set command parameter to 0 for sync
ir_loop_done  wrlong    outb, par
              jmp       #ir_loop

ir_rd_tim
'========
              movd      ir_rd_it, #ir_cog_t_tab
              mov       ir_temp+8, ir_tim_table
              mov       ir_temp+7, #20
ir_rd_tim_lp                        
ir_rd_it      rdword    0-0, ir_temp+8
              add       ir_rd_it,ir_dst_offs
              add       ir_temp+8,#2
              djnz      ir_temp+7, #ir_rd_tim_lp

              jmp       #ir_loop_done 

ir_setflag
'=========
              mov       ir_temp, ir_command
              shr       ir_temp, #8
              or        ir_flags, ir_temp
              jmp       #ir_loop_done
              

ir_clrflag
'=========
              mov       ir_temp, ir_command
              shr       ir_temp, #8
              andn      ir_flags, ir_temp
              jmp       #ir_loop_done
                            
ir_setpar
'========
              mov       ir_temp, ir_command
              ' calculate where to store the parameter
              movd      ir_store_par, ir_parameter
              mov       ir_temp+1, ir_command
              and       ir_temp+1, ir_b1
              shl       ir_temp+1, #1
              add       ir_store_par, ir_temp+1
              ' extract the 16 bit value
              shr       ir_temp, #16
ir_store_par  mov       0-0, ir_temp
              jmp       #ir_loop_done
              
' ir_translate
'
' This uses the timing-table created during read to translate the signal
' into a more readable version. Each different timing (allowing a deviation)
' is stored in that table sorted by time but separated by high- and low- times.
' During translation the signals timings are read again and the actual time is
' then searched in the timing table. The index gives the letter that's used in
' the translation string. Low times are converted to lower-case letters, high
' times are translated to upper-case letters.
'  
ir_translate
'===========  
              mov       dirb, #0
              mov       ir_temp, ir_smpl_size
              mov       ir_temp+1, ir_smpl_table
              mov       ir_temp+3, ir_trans_buf

ir_trans_lp   rdword    ir_temp+2, ir_temp+1
              add       ir_temp+1, #2

              call      #ir_find_trans
              xor       dirb, #1

              wrbyte    ir_temp+4, ir_temp+3
              add       ir_temp+3, #1
              djnz      ir_temp, #ir_trans_lp

              jmp       #ir_loop_done 

ir_find_trans
              movs      ir_ttf_read, #ir_cog_t_tab
              and       dirb, #1 WC
        if_C  add       ir_ttf_read, #10
              mov       ir_temp+7,#10
ir_ttf_read   mov       ir_temp+9, 0-0
              sub       ir_temp+9, ir_temp+2
              abs       ir_temp+9, ir_temp+9
              cmp       ir_temp+9, #3 WC,WZ
        if_BE jmp       #ir_found
              add       ir_ttf_read, #1
              djnz      ir_temp+7, #ir_ttf_read
ir_found
              mov       ir_temp+4, #"a"+10
              sub       ir_temp+4, ir_temp+7              
              and       dirb, #1 WC
        if_C  sub       ir_temp+4, #"a"-"A"
ir_find_trans_ret
              ret

ir_read
'======
              mov       ir_temp, ir_smpl_size
              mov       ir_temp+1, ir_smpl_table
              mov       dirb, #0

              'check the clear time table flag
              test      ir_flags, #IR_FLG_CLRTT WZ
        if_Z  jmp       #ir_still_wait
                
              movd      ir_cln_tim, #ir_cog_t_tab
              mov       ir_temp+2, #20

              ' clear the timing table
ir_cln_tim    mov       0-0, #0
              add       ir_cln_tim, ir_dst_offs
              djnz      ir_temp+2, #ir_cln_tim

              ' wait for the first low
ir_still_wait
              waitpeq   ir_in_mask, ir_in_mask
              waitpne   ir_in_mask, ir_in_mask
              mov       ir_tcnt, cnt
              add       ir_tcnt, ir_smpl_time
              ' double check the low
              test      ir_in_mask, ina WC
        if_c  jmp       #ir_still_wait

ir_read_0_prep
              mov       ir_bitcnt, #1
ir_read_0     waitcnt   ir_tcnt, ir_smpl_time
              test      ir_in_mask,ina WC
        if_nc add       ir_bitcnt,#1
        if_nc jmp       #ir_read_0

              wrword    ir_bitcnt, ir_temp+1
              call      #ir_add_tab
              xor       dirb, #1
              add       ir_temp+1, #2
              djnz      ir_temp, #ir_read_1_prep

              jmp       #ir_write_tim
              
ir_read_1_prep
              mov       ir_bitcnt, #1
ir_read_1     waitcnt   ir_tcnt, ir_smpl_time
              test      ir_in_mask, ina WC
        if_c  add       ir_bitcnt, #1
        if_c  jmp       #ir_read_1

              wrword    ir_bitcnt, ir_temp+1
              call      #ir_add_tab
              xor       dirb, #1
              add       ir_temp+1, #2
              djnz      ir_temp, #ir_read_0_prep

              ' jmp       #ir_write_tim

ir_write_tim
              'check the write time table flag
              test      ir_flags, #IR_FLG_WRTT WZ
        if_Z  jmp       #ir_loop_done

              movd      ir_write_back, #ir_cog_t_tab
              mov       ir_temp+8, ir_tim_table
              mov       ir_temp+7, #20
ir_write_tim_lp                        
ir_write_back wrword    0-0, ir_temp+8
              add       ir_write_back,ir_dst_offs
              add       ir_temp+8,#2
              djnz      ir_temp+7, #ir_write_tim_lp

              jmp       #ir_loop_done 
              
ir_add_tab
              movs      ir_tt_read, #ir_cog_t_tab
              and       dirb, #1 WC
        if_C  add       ir_tt_read, #10
              mov       ir_temp+7,#10
ir_tt_read    mov       ir_temp+9, 0-0 WZ
        if_z  jmp       #ir_tt_store
              mov       ir_temp+8, ir_temp+9
              sub       ir_temp+9, ir_bitcnt
              abs       ir_temp+9, ir_temp+9
              cmp       ir_temp+9, #3 WC,WZ
        if_BE jmp       #ir_add_tab_end '#ir_add_tab_ret
              cmp       ir_temp+8, ir_bitcnt WC,WZ
        if_B  jmp       #ir_add_next
              'jmp       #ir_add_next

ir_add_n_move movd      ir_mov_to, ir_tt_read
              movs      ir_mov_from, ir_tt_read
ir_mov_lp     mov       ir_temp+8, ir_bitcnt
ir_mov_from   mov       ir_temp+10, 0-0
ir_mov_to     mov       0-0, ir_temp+10
              add       ir_mov_from, #1
              add       ir_mov_to, ir_dst_offs
              djnz      ir_temp+7, #ir_mov_lp
              jmp       #ir_add_tab_end '#ir_add_tab_ret
               
ir_add_next   add       ir_tt_read, #1
              djnz      ir_temp+7, #ir_tt_read

ir_add_tab_end
              test      ir_flags, #IR_FLG_ONCE WZ
      if_z    jmp       #ir_add_tab_ret

              cmp       ir_gap, ir_bitcnt WC,WZ
      if_a    jmp       #ir_loop_done

ir_add_tab_ret
              ret

ir_tt_store
              movd      ir_tt_storeit, ir_tt_read
              nop
ir_tt_storeit mov       0-0, ir_bitcnt
              jmp       #ir_add_tab_end '#ir_add_tab_ret
                      
ir_send
'======
              mov       ir_temp, ir_smpl_table

              rdbyte    ir_temp+1, ir_temp WZ
              
              jmp       #ir_loop_done

ir_flags      long      0
ir_bitcnt     long      0
ir_tcnt       long      0
ir_tim_table  long      0
ir_smpl_table long      2
ir_smpl_size  long      0
ir_trans_buf  long      4
ir_smpl_time  long      4_000
ir_dst_offs   long      %1000000000
ir_b1         long      $0000_ff00

ir_parameter
ir_gap        long      $600

ir_in_mask    long      IR_InMask
ir_out_mask   long      IR_OutMask

' read the addresses of the buffers from parameter RAM
' the trans_buf must follow the smpl_buf, so that
' the smpl_size can be calculated
ir_temp
ir_init
'======
              add       ir_tim_table, par
              add       ir_smpl_table, par
              add       ir_trans_buf, par
              rdword    ir_tim_table, ir_tim_table
              rdword    ir_smpl_table, ir_smpl_table
              rdword    ir_trans_buf, ir_trans_buf
              mov       ir_smpl_size, ir_trans_buf
              sub       ir_smpl_size, ir_smpl_table
              shr       ir_smpl_size, #1
              ' set parameter to 0, so that SPIN can sync
              wrlong    outb, par
              jmp       #ir_loop

' time table in COG RAM
ir_cog_t_tab  long      10,0[19]
              
              fit       492                         